#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>

#define LOCK_FILE "lock.txt"

void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

/* Program demonstrira zaljucavanje fajla. */

int main(){
  int fd;
  struct flock lock;

  /* Otvaranje fajla. */
  if ( (fd = open (LOCK_FILE, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0)
    error_fatal ("open error");

  /* Popunjavamo polja podacima o katancu koji zelimo da postavimo. */
  lock.l_type = F_WRLCK;
  lock.l_whence = SEEK_SET;
  lock.l_start = 0;
  lock.l_len = 0;
  
  /* Pokusava se sa postavljanjem katanca. */
  if (fcntl (fd, F_SETLK, &lock) < 0){
    
    /* Obradjuje se slucaj kada postavljanje katanca nije uspelo jer
       drugi proces vec ima katanac. */
    if (errno == EACCES || errno == EAGAIN){
      printf ("Another instance already running.\n");
      exit (EXIT_SUCCESS);
    }
    /* Doslo je do neke druge vrste greske. */
    else
      error_fatal ("fcntl error");
  }
      
  /* Program se uspavljuje na 10 sekundi. Za ovo vreme treba pokrenuti
     drugu instancu programa i uveriti se da je zakljucavanje
     uspelo. */
  sleep (10);

  return 0;
}

